@@ -21,6 +21,9 @@
 
 #include "internal.h"
 
+#define DOTP(c) ((c) == 0x002E || (c) == 0x3002 ||	\
+		 (c) == 0xFF0E || (c) == 0xFF61)
+
 /* Core functions */
 
 /**
@@ -53,13 +56,12 @@
  *
  * Return value: Returns 0 on success, or an error code.
  */
-enum Idna_rc
-idna_to_ascii_4i (const uint32_t * in, size_t inlen, char *out,
-		  enum Idna_flags flags)
+int
+idna_to_ascii_4i (const uint32_t * in, size_t inlen, char *out, int flags)
 {
   size_t len, outlen;
   uint32_t *src;		/* XXX don't need to copy data? */
-  enum Idna_rc rc;
+  int rc;
 
   /*
    * ToASCII consists of the following steps:
@@ -239,10 +241,9 @@ step8:
 
 static int
 idna_to_unicode_internal (char *utf8in, size_t utf8len,
-			  uint32_t * out, size_t * outlen,
-			  enum Idna_flags flags)
+			  uint32_t * out, size_t * outlen, int flags)
 {
-  enum Idna_rc rc;
+  int rc;
   char tmpout[64];
 
   /*
@@ -359,11 +360,11 @@ step3:
  *               from this function, as checking it means breaking the
  *               standard.
  */
-enum Idna_rc
+int
 idna_to_unicode_44i (const uint32_t * in, size_t inlen,
-		     uint32_t * out, size_t * outlen, enum Idna_flags flags)
+		     uint32_t * out, size_t * outlen, int flags)
 {
-  enum Idna_rc rc;
+  int rc;
   size_t outlensave = *outlen;
   char *p;
 
@@ -402,37 +403,52 @@ idna_to_unicode_44i (const uint32_t * in, size_t inlen,
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_ascii_4z (const uint32_t * input, char **output,
-		  enum Idna_flags flags)
+int
+idna_to_ascii_4z (const uint32_t * input, char **output, int flags)
 {
   const uint32_t *start = input;
   const uint32_t *end = input;
   char buf[64];
   char *out = NULL;
-  enum Idna_rc rc;
+  int rc;
+
+  /* 1) Whenever dots are used as label separators, the following
+     characters MUST be recognized as dots: U+002E (full stop),
+     U+3002 (ideographic full stop), U+FF0E (fullwidth full stop),
+     U+FF61 (halfwidth ideographic full stop). */
+
+  if (input[0] == 0)
+    {
+      /* Handle implicit zero-length root label. */
+      *output = malloc (1);
+      if (!*output)
+	return IDNA_MALLOC_ERROR;
+      strcpy (*output, "");
+      return IDNA_SUCCESS;
+    }
+
+  if (DOTP (input[0]) && input[1] == 0)
+    {
+      /* Handle explicit zero-length root label. */
+      *output = malloc (2);
+      if (!*output)
+	return IDNA_MALLOC_ERROR;
+      strcpy (*output, ".");
+      return IDNA_SUCCESS;
+    }
 
   *output = NULL;
-
   do
     {
       end = start;
 
-      /* 1) Whenever dots are used as label separators, the following
-         characters MUST be recognized as dots: U+002E (full stop),
-         U+3002 (ideographic full stop), U+FF0E (fullwidth full stop),
-         U+FF61 (halfwidth ideographic full stop). */
-      for (; *end &&
-	   *end != 0x002E &&
-	   *end != 0x3002 && *end != 0xFF0E && *end != 0xFF61; end++)
+      for (; *end && !DOTP (*end); end++)
 	;
 
-      /* Handle empty trailing labels. The RFC is not clear on this,
-         the text that mandate this behaviour inside a parenthesis in
-         the terminology section. */
-      if (end == start && *end == '\0')
+      if (*end == '\0' && start == end)
 	{
-	  strcpy (buf, out ? "" : ".");
+	  /* Handle explicit zero-length root label. */
+	  buf[0] = '\0';
 	}
       else
 	{
@@ -451,9 +467,10 @@ idna_to_ascii_4z (const uint32_t * input, char **output,
 	}
       else
 	{
-	  out = (char *) strdup (buf);
+	  out = (char *) malloc (strlen (buf) + 1);
 	  if (!out)
 	    return IDNA_MALLOC_ERROR;
+	  strcpy (out, buf);
 	}
 
       start = end + 1;
@@ -477,12 +494,12 @@ idna_to_ascii_4z (const uint32_t * input, char **output,
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_ascii_8z (const char *input, char **output, enum Idna_flags flags)
+int
+idna_to_ascii_8z (const char *input, char **output, int flags)
 {
   uint32_t *ucs4;
   size_t ucs4len;
-  enum Idna_rc rc;
+  int rc;
 
   ucs4 = stringprep_utf8_to_ucs4 (input, -1, &ucs4len);
   if (!ucs4)
@@ -508,11 +525,11 @@ idna_to_ascii_8z (const char *input, char **output, enum Idna_flags flags)
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_ascii_lz (const char *input, char **output, enum Idna_flags flags)
+int
+idna_to_ascii_lz (const char *input, char **output, int flags)
 {
   char *utf8;
-  enum Idna_rc rc;
+  int rc;
 
   utf8 = stringprep_locale_to_utf8 (input);
   if (!utf8)
@@ -538,9 +555,8 @@ idna_to_ascii_lz (const char *input, char **output, enum Idna_flags flags)
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_unicode_4z4z (const uint32_t * input, uint32_t ** output,
-		      enum Idna_flags flags)
+int
+idna_to_unicode_4z4z (const uint32_t * input, uint32_t ** output, int flags)
 {
   const uint32_t *start = input;
   const uint32_t *end = input;
@@ -548,7 +564,7 @@ idna_to_unicode_4z4z (const uint32_t * input, uint32_t ** output,
   size_t buflen;
   uint32_t *out = NULL;
   size_t outlen = 0;
-  enum Idna_rc rc;
+  int rc;
 
   *output = NULL;
 
@@ -556,13 +572,7 @@ idna_to_unicode_4z4z (const uint32_t * input, uint32_t ** output,
     {
       end = start;
 
-      /* 1) Whenever dots are used as label separators, the following
-         characters MUST be recognized as dots: U+002E (full stop),
-         U+3002 (ideographic full stop), U+FF0E (fullwidth full stop),
-         U+FF61 (halfwidth ideographic full stop). */
-      for (; *end &&
-	   *end != 0x002E &&
-	   *end != 0x3002 && *end != 0xFF0E && *end != 0xFF61; end++)
+      for (; *end && !DOTP (*end); end++)
 	;
 
       buflen = end - start;
@@ -613,13 +623,12 @@ idna_to_unicode_4z4z (const uint32_t * input, uint32_t ** output,
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_unicode_8z4z (const char *input, uint32_t ** output,
-		      enum Idna_flags flags)
+int
+idna_to_unicode_8z4z (const char *input, uint32_t ** output, int flags)
 {
   uint32_t *ucs4;
   size_t ucs4len;
-  enum Idna_rc rc;
+  int rc;
 
   ucs4 = stringprep_utf8_to_ucs4 (input, -1, &ucs4len);
   if (!ucs4)
@@ -644,11 +653,11 @@ idna_to_unicode_8z4z (const char *input, uint32_t ** output,
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_unicode_8z8z (const char *input, char **output, enum Idna_flags flags)
+int
+idna_to_unicode_8z8z (const char *input, char **output, int flags)
 {
   uint32_t *ucs4;
-  enum Idna_rc rc;
+  int rc;
 
   rc = idna_to_unicode_8z4z (input, &ucs4, flags);
   *output = stringprep_ucs4_to_utf8 (ucs4, -1, NULL, NULL);
@@ -674,11 +683,11 @@ idna_to_unicode_8z8z (const char *input, char **output, enum Idna_flags flags)
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_unicode_8zlz (const char *input, char **output, enum Idna_flags flags)
+int
+idna_to_unicode_8zlz (const char *input, char **output, int flags)
 {
   char *utf8;
-  enum Idna_rc rc;
+  int rc;
 
   rc = idna_to_unicode_8z8z (input, &utf8, flags);
   *output = stringprep_utf8_to_locale (utf8);
@@ -705,11 +714,11 @@ idna_to_unicode_8zlz (const char *input, char **output, enum Idna_flags flags)
  *
  * Return value: Returns IDNA_SUCCESS on success, or error code.
  **/
-enum Idna_rc
-idna_to_unicode_lzlz (const char *input, char **output, enum Idna_flags flags)
+int
+idna_to_unicode_lzlz (const char *input, char **output, int flags)
 {
   char *utf8;
-  enum Idna_rc rc;
+  int rc;
 
   utf8 = stringprep_locale_to_utf8 (input);
   if (!utf8)
@@ -720,3 +729,49 @@ idna_to_unicode_lzlz (const char *input, char **output, enum Idna_flags flags)
 
   return rc;
 }
+
+/**
+ * IDNA_ACE_PREFIX
+ *
+ * The IANA allocated prefix to use for IDNA. "xn--"
+ */
+
+/**
+ * Idna_rc:
+ * @IDNA_SUCCESS: Successful operation.  This value is guaranteed to
+ *   always be zero, the remaining ones are only guaranteed to hold
+ *   non-zero values, for logical comparison purposes.
+ * @IDNA_STRINGPREP_ERROR:  Error during string preparation.
+ * @IDNA_PUNYCODE_ERROR: Error during punycode operation.
+ * @IDNA_CONTAINS_LDH: For IDNA_USE_STD3_ASCII_RULES, indicate that
+ *   the string contains LDH ASCII characters.
+ * @IDNA_CONTAINS_MINUS: For IDNA_USE_STD3_ASCII_RULES, indicate that
+ *   the string contains a leading or trailing hyphen-minus (U+002D).
+ * @IDNA_INVALID_LENGTH: The final output string is not within the
+ *   (inclusive) range 1 to 63 characters.
+ * @IDNA_NO_ACE_PREFIX: The string does not contain the ACE prefix
+ *   (for ToUnicode).
+ * @IDNA_ROUNDTRIP_VERIFY_ERROR: The ToASCII operation on output
+ *   string does not equal the input.
+ * @IDNA_CONTAINS_ACE_PREFIX: The input contains the ACE prefix (for
+ *   ToASCII).
+ * @IDNA_ICONV_ERROR: Could not convert string in locale encoding.
+ * @IDNA_MALLOC_ERROR: Could not allocate buffer (this is typically a
+ *   fatal error).
+ *
+ * Enumerated return codes of idna_to_ascii_4i(),
+ * idna_to_unicode_44i() functions (and functions derived from those
+ * functions).  The value 0 is guaranteed to always correspond to
+ * success.
+ */
+
+
+/**
+ * Idna_flags:
+ * @IDNA_ALLOW_UNASSIGNED: Don't reject strings containing unassigned
+ *   Unicode code points.
+ * @IDNA_USE_STD3_ASCII_RULES: Validate strings according to STD3
+ *   rules (i.e., normal host name rules).
+ *
+ * Flags to pass to idna_to_ascii_4i(), idna_to_unicode_44i() etc.
+ */
