@@ -31,7 +31,7 @@ extern "C"
 #include <unistd.h>		/* ssize_t */
 #include <idn-int.h>		/* uint32_t */
 
-#define STRINGPREP_VERSION "0.3.1"
+#define STRINGPREP_VERSION "0.3.2"
 
 /* Error codes. */
   typedef enum
@@ -142,6 +142,7 @@ extern "C"
 
   extern Stringprep_profile stringprep_saslprep[];
   extern Stringprep_profile stringprep_plain[];
+  extern Stringprep_profile stringprep_trace[];
 
 #define stringprep_plain(in, maxlen)		\
   stringprep(in, maxlen, 0, stringprep_plain)
@@ -177,7 +178,7 @@ extern "C"
 			 Stringprep_profile_flags flags,
 			 Stringprep_profile * profile);
 
-  extern int stringprep_profile (char *in,
+  extern int stringprep_profile (const char *in,
 				 char **out,
 				 char *profile,
 				 Stringprep_profile_flags flags);
