@@ -87,7 +87,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; p[i]; i++)
-		fprintf (stderr, "input[%d] = U+%04x\n", i, p[i] & 0xFFFF);
+		fprintf (stderr, "input[%d] = U+%0.4x\n", i, p[i] & 0xFF);
 	    }
 
 	  rc = stringprep_profile (p, &r,
@@ -106,7 +106,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; r[i]; i++)
-		fprintf (stderr, "output[%d] = U+%04x\n", i, r[i] & 0xFFFF);
+		fprintf (stderr, "output[%d] = U+%0.4x\n", i, r[i] & 0xFF);
 	    }
 
 	  p = stringprep_utf8_to_locale (r);
@@ -148,7 +148,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; i < len; i++)
-		fprintf (stderr, "input[%d] = U+%04x\n", i, q[i] & 0xFFFF);
+		fprintf (stderr, "input[%d] = U+%0.4x\n", i, q[i]);
 	    }
 
 	  len2 = BUFSIZ;
@@ -205,7 +205,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; i < len; i++)
-		fprintf (stderr, "output[%d] = U+%04x\n", i, q[i] & 0xFFFF);
+		fprintf (stderr, "output[%d] = U+%0.4x\n", i, q[i]);
 	    }
 
 	  q[len] = 0;
@@ -255,7 +255,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; q[i]; i++)
-		fprintf (stderr, "input[%d] = U+%04x\n", i, q[i] & 0xFFFF);
+		fprintf (stderr, "input[%d] = U+%0.4x\n", i, q[i]);
 	    }
 
 	  rc = idna_to_ascii_4z (q, &r,
@@ -298,7 +298,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; q[i]; i++)
-		fprintf (stderr, "input[%d] = U+%04x\n", i, q[i] & 0xFFFF);
+		fprintf (stderr, "input[%d] = U+%0.4x\n", i, q[i]);
 	    }
 	  free (q);
 
@@ -319,7 +319,7 @@ main (int argc, char *argv[])
 	    {
 	      size_t i;
 	      for (i = 0; q[i]; i++)
-		fprintf (stderr, "output[%d] = U+%04x\n", i, q[i] & 0xFFFF);
+		fprintf (stderr, "output[%d] = U+%0.4x\n", i, q[i]);
 	    }
 
 	  p = stringprep_ucs4_to_utf8 (q, -1, NULL, NULL);
